package sheet05;

import jast.config.ADAPTER;
import jast.config.IDENTITY;
import jast.config.STORMNAME;

import java.util.HashSet;
import java.util.Set;

import Ice.Application;
import Ice.Identity;
import Ice.ObjectAdapter;
import Ice.ObjectPrx;
import Ice.Properties;
import Ice.Util;
import IceStorm.NoSuchTopic;
import IceStorm.TopicExists;
import IceStorm.TopicManagerPrx;
import IceStorm.TopicManagerPrxHelper;
import IceStorm.TopicPrx;

/**
 * A main class that uses the Ice.Application framework to simplify the process
 * of connecting to the Ice runtime.
 * 
 * @author mef
 */
public class ObjectInventoryApp extends Application {
	
    Set<TopicPrx> subscribedTopics = new HashSet<TopicPrx>();
    
    private void subscribeToTopic(String topicName, TopicManagerPrx tmPrx, ObjectPrx myPrx) {
        TopicPrx topicPrx;
        try {
            topicPrx = tmPrx.retrieve(topicName);
        } catch(NoSuchTopic ex) {
            try {
                topicPrx = tmPrx.create(topicName);
            } catch(TopicExists ex2) {
                System.err.println("Unable to retrieve or create topic '" + topicName + "'");
                return;
            }
        }
        try {
            topicPrx.subscribe(null, myPrx);
        } catch(Exception e) {
            System.err.println("Already subscribed to topic '" + topicName + "'");
            return;
        }
        System.err.println("Subscribed " + myPrx + " to " + topicPrx);
        subscribedTopics.add(topicPrx);
    }

    /**
     * Does the actual work of connecting to Ice and activating the server.
     * 
     * @param args
     *            The command-line arguments, with all Ice-specific data removed
     * @see Ice.Application#run(java.lang.String[])
     */
    @Override
    public int run(String[] args) {
        // Connect to the Ice runtime
        Properties props = communicator().getProperties();
        ObjectAdapter adapter = communicator().createObjectAdapter(ADAPTER.value);
        Identity id = Util.stringToIdentity(props.getProperty(IDENTITY.value));

        ObjectInventoryI impl = new ObjectInventoryI();
        ObjectPrx myPrx = adapter.add(impl, id);

        // Subscribe to all of the topics
        String iceStormName = props.getProperty(STORMNAME.value);
        TopicManagerPrx tmPrx = TopicManagerPrxHelper.checkedCast(communicator().stringToProxy(
                iceStormName + "/TopicManager"));
        
        for(String iceId : impl.ice_ids()) {
            if(iceId.startsWith("::jast::listener::ObjectRecognitionListener")) {
                subscribeToTopic(iceId, tmPrx, myPrx);
            }
        }
        
        // Create a factory for Items ...
        adapter.activate();

        // Wait until we're disconnected from the runtime and then return
        communicator().waitForShutdown();
        
        // Unsubscribe from everything we subscribed to
        for(TopicPrx topicPrx : subscribedTopics) {
            System.err.println("Unsubscribing " + myPrx + " from " + topicPrx);
            topicPrx.unsubscribe(myPrx);
        }
        
        return 0;
    }

    /**
     * The actual main method of this server. Delegates everything to the
     * Ice.Application's <code>main</code> method.
     * 
     * @param args
     *            The command-line arguments
     * @see Ice.Application#main(String, String[])
     */
    public static void main(String[] args) {
        ObjectInventoryApp app = new ObjectInventoryApp();
        System.exit(app.main("ObjectInventory", args));
    }

}
